package com.bcxin.tenant.domain.services.impls;

import cn.hutool.core.collection.CollectionUtil;
import com.bcxin.Infrastructures.TenantContext;
import com.bcxin.Infrastructures.TenantUserContext;
import com.bcxin.Infrastructures.UnitWork;
import com.bcxin.Infrastructures.entities.OperatorValueType;
import com.bcxin.Infrastructures.enums.InvitedStatus;
import com.bcxin.Infrastructures.enums.InvitedType;
import com.bcxin.Infrastructures.exceptions.NotFoundTenantException;
import com.bcxin.Infrastructures.exceptions.NotSupportTenantException;
import com.bcxin.Infrastructures.models.CredentialCheckInfo;
import com.bcxin.Infrastructures.models.CredentialModel;
import com.bcxin.Infrastructures.utils.AuthUtil;
import com.bcxin.api.interfaces.identities.requests.InvitedToJoinQueuesRequest;
import com.bcxin.api.interfaces.security.AttendSitePersonRpcProvider;
import com.bcxin.api.interfaces.tenants.EmployeeRpcProvider;
import com.bcxin.api.interfaces.tenants.InviteAttendSiteRpcProvider;
import com.bcxin.api.interfaces.tenants.InviteDepartRpcProvider;
import com.bcxin.api.interfaces.tenants.requests.employees.EmployeeRequest;
import com.bcxin.api.interfaces.tenants.responses.InviteAttendSiteResponse;
import com.bcxin.api.interfaces.tenants.responses.InviteDepartResponse;
import com.bcxin.api.interfaces.tenants.responses.InvitedToJoinQueuesHandleResponse;
import com.bcxin.tenant.domain.entities.DepartmentEntity;
import com.bcxin.tenant.domain.entities.InvitedToJoinQueuesEntity;
import com.bcxin.tenant.domain.entities.SharedMessagesEntity;
import com.bcxin.tenant.domain.readers.TenantDbReader;
import com.bcxin.tenant.domain.repositories.DepartmentRepository;
import com.bcxin.tenant.domain.repositories.InvitedToJoinQueuesRepository;
import com.bcxin.tenant.domain.repositories.SharedMessagesRepository;
import com.bcxin.tenant.domain.services.EmployeeService;
import com.bcxin.tenant.domain.services.InvitedToJoinQueuesService;
import com.bcxin.tenant.domain.services.commands.CreateEmployeeRequestCommand;
import com.bcxin.tenant.domain.services.commands.InvitedToJoinQueuesCommand;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.sql.Timestamp;
import java.time.Instant;
import java.util.*;
import java.util.concurrent.atomic.AtomicReference;

@Service
public class InvitedToJoinQueuesServiceImpl implements InvitedToJoinQueuesService {

    private final InvitedToJoinQueuesRepository invitedToJoinQueuesRepository;

    private final InviteDepartRpcProvider inviteDepartRpcProvider;

    private final DepartmentRepository departmentRepository;

    private final InviteAttendSiteRpcProvider inviteAttendSiteRpcProvider;

    private final UnitWork unitWork;

    private final TenantDbReader dbReader;

    private final EmployeeRpcProvider employeeRpcProvider;

    private final AttendSitePersonRpcProvider attendSitePersonRpcProvider;

    private final SharedMessagesRepository sharedMessagesRepository;

    private final EmployeeService employeeService;

    public InvitedToJoinQueuesServiceImpl(InvitedToJoinQueuesRepository invitedToJoinQueuesRepository, InviteDepartRpcProvider inviteDepartRpcProvider,
                                          DepartmentRepository departmentRepository, InviteAttendSiteRpcProvider inviteAttendSiteRpcProvider, UnitWork unitWork, TenantDbReader dbReader,
                                          EmployeeRpcProvider employeeRpcProvider, AttendSitePersonRpcProvider attendSitePersonRpcProvider, SharedMessagesRepository sharedMessagesRepository, EmployeeService employeeService) {
        this.invitedToJoinQueuesRepository = invitedToJoinQueuesRepository;
        this.inviteDepartRpcProvider = inviteDepartRpcProvider;
        this.departmentRepository = departmentRepository;
        this.inviteAttendSiteRpcProvider = inviteAttendSiteRpcProvider;
        this.unitWork = unitWork;
        this.dbReader = dbReader;
        this.employeeRpcProvider = employeeRpcProvider;
        this.attendSitePersonRpcProvider = attendSitePersonRpcProvider;
        this.sharedMessagesRepository = sharedMessagesRepository;
        this.employeeService = employeeService;
    }

    @Override
    @Transactional
    public void dispatch(InvitedToJoinQueuesCommand invitedToJoinQueuesCommand) {
        DepartmentEntity department = departmentRepository.findById(invitedToJoinQueuesCommand.getDepartmentId()).orElse(null);
        if (department == null || department.isDeleted()) {
            throw new NotFoundTenantException("该部门信息无效");
        }
        OperatorValueType operatorValueType = null;
        String organizationId = null;
        String attendanceSiteName = null;
        String attendanceSiteId = null;
        if (invitedToJoinQueuesCommand.getInvitedType() == InvitedType.InviteDepart) {
            InviteDepartResponse inviteDepart = inviteDepartRpcProvider.getUnOvertime(invitedToJoinQueuesCommand.getInviteId());
            if (inviteDepart == null) {
                throw new NotFoundTenantException("该邀请不存在或者已截止！");
            }
            if (!invitedToJoinQueuesCommand.getDepartmentId().equals(inviteDepart.getDepartId())) {
                throw new NotSupportTenantException("部门与邀请不符！");
            }
            if (!invitedToJoinQueuesCommand.getOccupationType().equals(inviteDepart.getOccupationType())) {
                throw new NotSupportTenantException("职业类型与邀请不符！");
            }
            operatorValueType = inviteDepart.getCreator();
            organizationId = inviteDepart.getOrganizationId();
        } else if (invitedToJoinQueuesCommand.getInvitedType() == InvitedType.InviteAttend) {
            InviteAttendSiteResponse inviteAttendSite = inviteAttendSiteRpcProvider.getUnOvertime(invitedToJoinQueuesCommand.getInviteId());
            if (inviteAttendSite == null) {
                throw new NotFoundTenantException("该邀请不存在或者已截止！");
            }
            if (!invitedToJoinQueuesCommand.getDepartmentId().equals(inviteAttendSite.getDepartId())) {
                throw new NotSupportTenantException("部门与邀请不符！");
            }
            if (!invitedToJoinQueuesCommand.getOccupationType().equals(inviteAttendSite.getOccupationType())) {
                throw new NotSupportTenantException("职业类型与邀请不符！");
            }
            operatorValueType = inviteAttendSite.getCreator();
            organizationId = inviteAttendSite.getOrganizationId();
            attendanceSiteName = inviteAttendSite.getAttendSiteName();
            attendanceSiteId = inviteAttendSite.getAttendSiteId();
        } else {
            throw new NotSupportTenantException("邀请类型不符！");
        }
        AtomicReference<InvitedToJoinQueuesEntity> invitedAto = new AtomicReference<>();
        final OperatorValueType finalOperatorValueType = operatorValueType;
        final String finalOrganizationId = organizationId;
        final String finalAttendanceSiteName = attendanceSiteName;
        final String finalAttendanceSiteId = attendanceSiteId;
        this.unitWork.executeTran(() -> {
            invitedAto.set(
                    InvitedToJoinQueuesEntity.create(invitedToJoinQueuesCommand.getName(),
                            invitedToJoinQueuesCommand.getTelephone(),
                            invitedToJoinQueuesCommand.getCredentialType(),
                            invitedToJoinQueuesCommand.getCredentialNumber(),
                            finalOperatorValueType,
                            invitedToJoinQueuesCommand.getInvitedType(),
                            invitedToJoinQueuesCommand.getDepartmentId(),
                            department.getName(),
                            finalOrganizationId,
                            invitedToJoinQueuesCommand.getOccupationType(),
                            finalAttendanceSiteName,
                            finalAttendanceSiteId,
                            invitedToJoinQueuesCommand.getFrontPhoto(),
                            invitedToJoinQueuesCommand.getReversePhoto(),
                            invitedToJoinQueuesCommand.getHeadPhoto(),
                            invitedToJoinQueuesCommand.getValidDateFrom(),
                            invitedToJoinQueuesCommand.getValidDateTo(),
                            invitedToJoinQueuesCommand.getAddress(),
                            invitedToJoinQueuesCommand.getCertificateImage()));
            invitedToJoinQueuesRepository.save(invitedAto.get());
        });
    }

    @Override
    @Transactional
    public void update(InvitedToJoinQueuesRequest request) {
        //登录账号
        TenantUserContext.UserModel currentUser = TenantContext.getInstance().getUserContext().get();
        //找出要入职的人员信息
        Collection<InvitedToJoinQueuesEntity> invitedToJoinQueuesEntities = this.invitedToJoinQueuesRepository.findByIds(request.getIds());
        for (InvitedToJoinQueuesEntity invitedToJoinQueuesEntity : invitedToJoinQueuesEntities) {
            //需要没有处理过的，才能操作
            if (!InvitedStatus.Pending.equals(request.getInvitedStatus())
                    && (invitedToJoinQueuesEntity.getInvitedStatus() == null
                    || InvitedStatus.Pending.equals(invitedToJoinQueuesEntity.getInvitedStatus()))) {
                //请求的处理状态不是待处理 && （ 处理状态为空 || 待处理 ）
                if (InvitedType.InviteGovDeclare.equals(invitedToJoinQueuesEntity.getInvitedType()) && InvitedStatus.Agree.equals(request.getInvitedStatus())) {
                    //邀请类型是政务网申报 && 确认入职
                    if (StringUtils.isBlank(request.getDepartId())) {
                        throw new NotFoundTenantException("部门id不能为空");
                    }
                    DepartmentEntity department = departmentRepository.findById(request.getDepartId()).orElse(null);
                    if (department == null || department.isDeleted()) {
                        throw new NotFoundTenantException("该部门信息无效");
                    }
                    //填入部门信息
                    invitedToJoinQueuesEntity.changeDepart(department.getId(), department.getName());
                }

                String messageContent = "";
                if (InvitedStatus.Agree.equals(request.getInvitedStatus())) {
                    //短信通知内容
                    messageContent = "【百川信】您的百保盾账号已创建，账号为您的身份证号，初始密码为账号后六位，请点击下载地址下载App进行登录。下载地址:http://www.baibaodun.cn/a";
                } else if (InvitedStatus.Refuse.equals(request.getInvitedStatus())) {
                    //拒绝入职
                    //短信通知内容
                    messageContent = invitedToJoinQueuesEntity.getName().concat("，您提交的保安员证核发申报失败，请重新提交申请！");
                }
                String finalMessageContent = messageContent;
                this.unitWork.executeTran(() -> {
                    invitedToJoinQueuesEntity.change(request.getInvitedStatus(), currentUser.getEmployeeId(), currentUser.getName());
                    this.invitedToJoinQueuesRepository.save(invitedToJoinQueuesEntity);

                    if(InvitedStatus.Agree.equals(request.getInvitedStatus())) {
                        CreateEmployeeRequestCommand createEmployeeRequestCommand =
                                CreateEmployeeRequestCommand.create(invitedToJoinQueuesEntity.getOrganizationId(),
                                        invitedToJoinQueuesEntity.getName(),
                                        invitedToJoinQueuesEntity.getTelephone(),
                                        null,
                                        invitedToJoinQueuesEntity.getDepartmentId(),
                                        invitedToJoinQueuesEntity.getOccupationType(),
                                        null,
                                        Timestamp.from(Instant.now()),
                                        invitedToJoinQueuesEntity.getCredentialType(),
                                        invitedToJoinQueuesEntity.getCredentialNumber(),
                                        AuthUtil.getCurrentOperator(),
                                        CredentialCheckInfo.create(true,
                                                invitedToJoinQueuesEntity.getFrontPhoto(),
                                                invitedToJoinQueuesEntity.getReversePhoto(),
                                                invitedToJoinQueuesEntity.getHeadPhoto(),
                                                invitedToJoinQueuesEntity.getValidDateFrom(),
                                                invitedToJoinQueuesEntity.getValidDateTo(),
                                                invitedToJoinQueuesEntity.getAddress(),
                                                invitedToJoinQueuesEntity.getCertificateImage()));
                        String employeeId = this.employeeService.dispatch(createEmployeeRequestCommand);

                        if (InvitedType.InviteAttend.equals(invitedToJoinQueuesEntity.getInvitedType())) {
                            InviteAttendSiteResponse inviteAttendSite = new InviteAttendSiteResponse();
                            inviteAttendSite.setAttendSiteId(invitedToJoinQueuesEntity.getAttendanceSiteId());
                            inviteAttendSite.setDepartId(invitedToJoinQueuesEntity.getDepartmentId());
                            inviteAttendSite.setDepartName(invitedToJoinQueuesEntity.getDepartmentName());
                            inviteAttendSite.setOrganizationId(invitedToJoinQueuesEntity.getOrganizationId());
                            EmployeeRequest employeeRequest = new EmployeeRequest();
                            employeeRequest.setName(invitedToJoinQueuesEntity.getName());
                            employeeRequest.setTelephone(invitedToJoinQueuesEntity.getTelephone());
                            employeeRequest.setDepartId(invitedToJoinQueuesEntity.getDepartmentId());
                            employeeRequest.setOccupationType(invitedToJoinQueuesEntity.getOccupationType());
                            employeeRequest.setCredential(CredentialModel.create(invitedToJoinQueuesEntity.getCredentialType(), invitedToJoinQueuesEntity.getCredentialNumber()));
                            employeeRequest.setPosition(null);
                            employeeRequest.setHiredDate(Timestamp.from(Instant.now()));
                            this.attendSitePersonRpcProvider.create(inviteAttendSite, employeeRequest, employeeId);
                        }
                    }

                    //保存短信通知
                    SharedMessagesEntity messages = new SharedMessagesEntity();
                    messages.setId(UUID.randomUUID().toString());
                    messages.setBusinessNumber(messages.getId());
                    messages.setUniqueId(messages.getId());
                    messages.setContent(finalMessageContent);
                    messages.setCreatedTime(new Date());
                    messages.setMessageType(1);
                    messages.setReceiver(invitedToJoinQueuesEntity.getTelephone());
                    messages.setReceiverType(1);
                    messages.setSenderId(messages.getId());
                    messages.setStatus(1);
                    messages.setTitle("百保盾");
                    sharedMessagesRepository.save(messages);
                });
            }
        }
    }

    /**
     * description：根据证件号码，查询邀请入职列表，取最新的那条
     * author：linchunpeng
     * date：2024/4/18
     */
    @Override
    public List<InvitedToJoinQueuesHandleResponse> findListByCredentialNumbers(List<String> credentialNumberList) {
        List<InvitedToJoinQueuesHandleResponse> resultList = new ArrayList<>();
        if (CollectionUtil.isNotEmpty(credentialNumberList)) {
            List<InvitedToJoinQueuesEntity> list = invitedToJoinQueuesRepository.findByCredentialNumbers(credentialNumberList);
            if (CollectionUtil.isNotEmpty(list)) {
                Set<String> uniqueSet = new HashSet<>();
                for (InvitedToJoinQueuesEntity invitedToJoinQueuesEntity : list) {
                    if (uniqueSet.add(invitedToJoinQueuesEntity.getCredentialNumber())) {
                        resultList.add(InvitedToJoinQueuesHandleResponse.create(invitedToJoinQueuesEntity.getCredentialNumber(), invitedToJoinQueuesEntity.getInvitedStatus()));
                    }
                }
            }
        }
        return resultList;
    }
}
